package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ConceptWithOrderRes;
import com.diagbot.dto.PartDTO;
import com.diagbot.dto.PartSymptomDTO;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.entity.Symptom;
import com.diagbot.entity.wrapper.ConceptWrapper;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconRSTypeEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.PartSymptomVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description: 部位facade
 * @Author: ztg
 * @Date: 2018/10/23 16:33
 */
@Component
public class PartFacade {

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    ConceptDetailFacade conceptDetailFacade;

    /**
     * 根据已选部位返回对应的症状
     *
     * @param partSymptomVO 参数
     * @return 部位症状关联信息
     */
    public List<PartSymptomDTO> getSymptomByPartName(PartSymptomVO partSymptomVO) {
        List<String> partList = partSymptomVO.getPartList();
        if (ListUtil.isEmpty(partList)) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择部位");
        }
        if (partSymptomVO.getSexType() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择性别");
        }
        if (partSymptomVO.getAge() == null) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL,
                    "请选择年龄");
        }
        if (partSymptomVO.getAge().intValue() < 0 || partSymptomVO.getAge().intValue() > 200) {
            throw new CommonException(CommonErrorCode.PARAM_IS_ERROR,
                    "超出年龄范围【0 — 200】");
        }
        List<PartSymptomDTO> res = new ArrayList<>();

        // 获取部位信息
        ConceptWrapper conceptWrapper = new ConceptWrapper();
        conceptWrapper.setStartNameList(partList);
        conceptWrapper.setStartType(LexiconTypeEnum.BODYPART.getKey());
        conceptWrapper.setRelationType(LexiconRSTypeEnum.ORDER_BY.getKey());
        conceptWrapper.setEndType(LexiconTypeEnum.BODYPART.getKey());
        conceptWrapper.setEndSex(partSymptomVO.getSexType());
        conceptWrapper.setEndAge(partSymptomVO.getAge());
        List<ConceptWithOrderRes> list = conceptFacade.getConceptWithOrder(conceptWrapper);
        // 设置一级部位
        for (ConceptWithOrderRes concept : list) {
            if (concept.getStartId().equals(concept.getEndId())) { //startId 和 endId相同标识一级部位
                PartSymptomDTO dto = new PartSymptomDTO();
                dto.setName(concept.getStartName());
                dto.setConceptId(concept.getStartId());
                res.add(dto);
            }
        }

        // 设置二级部位
        List<String> secPart = new ArrayList<>();
        for (PartSymptomDTO dto : res) {
            List<PartDTO> partDTO = new ArrayList<>();
            for (ConceptWithOrderRes concept : list) {
                if (dto.getName().equals(concept.getStartName())
                        && !concept.getStartName().equals(concept.getEndName())) {
                    PartDTO dto1 = new PartDTO();
                    dto1.setName(concept.getEndName());
                    dto1.setConceptId(concept.getEndId());
                    secPart.add(concept.getEndName());
                    partDTO.add(dto1);
                }
            }
            dto.setPartDTO(partDTO);
        }

        // 获取二级部位下症状信息
        ConceptWrapper wrapper = new ConceptWrapper();
        wrapper.setStartNameList(secPart);
        wrapper.setStartType(LexiconTypeEnum.BODYPART.getKey());
        wrapper.setRelationType(LexiconRSTypeEnum.ORDER_BY.getKey());
        wrapper.setEndType(LexiconTypeEnum.SYMPTOM.getKey());
        wrapper.setEndSex(partSymptomVO.getSexType());
        wrapper.setEndAge(partSymptomVO.getAge());
        List<ConceptWithOrderRes> sympton = conceptFacade.getConceptWithOrder(wrapper);

        // 出参转化
        List<Symptom> symptomList = new ArrayList<>();
        for (ConceptWithOrderRes conceptWithOrderRes : sympton) {
            Symptom bean = new Symptom();
            bean.setName(conceptWithOrderRes.getEndName());
            bean.setConceptId(conceptWithOrderRes.getEndId());
            bean.setPartConceptId(conceptWithOrderRes.getStartId());
            symptomList.add(bean);
        }

        // 添加症状的简述信息
        List<ConceptDetail> conceptDetailList = conceptDetailFacade.list(
                new QueryWrapper<ConceptDetail>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("concept_id", symptomList.stream().map(row -> row.getConceptId()).distinct().collect(Collectors.toList()))
                .apply("find_in_set({0},position)", 7));
        Map<Long, String> map = conceptDetailList.stream().collect(Collectors.toMap(row->row.getConceptId(), row->row.getText()));
        System.out.println(list);
        for (Symptom symptom : symptomList) {
            if (map.get(symptom.getConceptId()) != null) {
                symptom.setDesc(map.get(symptom.getConceptId()));
            }
        }

        Map<Long, List<Symptom>> keyMap = EntityUtil.makeEntityListMap(symptomList, "partConceptId");

        // 设置二级部位对应的症状
        for (PartSymptomDTO dto : res) {
            for (PartDTO partDTO : dto.getPartDTO()) {
                if (keyMap.get(partDTO.getConceptId()) != null) { // 去除null数据
                    partDTO.setSymptomList(keyMap.get(partDTO.getConceptId()));
                }
            }
        }

        // 如二级部位无关联的症状，则删除二级部位
        for (PartSymptomDTO dto : res) {
            for (int i = 0; i < dto.getPartDTO().size(); i++) {
                PartDTO partDTO = dto.getPartDTO().get(i);
                if (keyMap.get(partDTO.getConceptId()) != null) { // 去除null数据
                    partDTO.setSymptomList(keyMap.get(partDTO.getConceptId()));
                }
                if (ListUtil.isEmpty(partDTO.getSymptomList())) {
                    dto.getPartDTO().remove(i--);
                }
            }
        }
        return res;
    }


}
